--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    12:49:24 03/07/06
-- Design Name:    
-- Module Name:    Loopback - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Loopback is
    Port ( mclk : in std_logic;
           rst : in std_logic;
           strt : in std_logic;
           pass : out std_logic := '0';
           fail : out std_logic := '0';
           TstOut : out std_logic_vector(8 downto 0);
           TstIn : in std_logic_vector(8 downto 0));
end Loopback;

architecture Behavioral of Loopback is

------------------------------------------------------------------------
--  Constant Declarations
------------------------------------------------------------------------

	-- The following constants define state codes for the EPP port interface
	-- state machine. The high order bits of the state number give a unique
	-- state identifier. The low order bits are the state machine outputs for
	-- that state. This type of state machine implementation uses no
	-- combination logic to generate outputs which should produce glitch
	-- free outputs.
	constant	stInit		: std_logic_vector(7 downto 0) := "0000" & "0000";
	constant	stTest1		: std_logic_vector(7 downto 0) := "0001" & "0100";
	constant	stTest2		: std_logic_vector(7 downto 0) := "0010" & "0001";
	constant	stTest3		: std_logic_vector(7 downto 0) := "0011" & "0010";
	constant	stTest4		: std_logic_vector(7 downto 0) := "0100" & "0011";
	constant	stPass		: std_logic_vector(7 downto 0) := "0101" & "1000";
	constant	stFail		: std_logic_vector(7 downto 0) := "0110" & "0001";

------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------

	-- State machine current state register
	signal	stCur				: std_logic_vector(7 downto 0) := stInit;
	signal	stNext			: std_logic_vector(7 downto 0);
	signal	clkMain			: std_logic;
	signal 	ctrlRst			: std_logic;
	signal 	ctrlStrt			: std_logic;
	signal 	ctrlPass			: std_logic;
	signal 	ctrlFail			: std_logic;
   signal	vectTstOut		: std_logic_vector(8 downto 0);
   signal 	vectTstIn 		: std_logic_vector(8 downto 0);

begin

	clkMain <= mclk;
	ctrlRst <= rst;
	ctrlStrt <= strt;
	vectTstIn <= TstIn;	

	-- This process moves the state machine to the next state
	-- on each clock cycle
	process (clkMain)
		begin
			if clkMain = '1' and clkMain'Event then
				stCur <= stNext;
			end if;
		end process;

	-- This process determines the next state machine state based
	-- on the current state and the state machine inputs.
	process (stCur, stNext, ctrlRst, ctrlStrt, vectTstIn)
		begin
			case stCur is

				when stInit =>
					if ctrlStrt = '1' then
						stNext <= stTest1;
					else
						stNext <= stInit;
					end if;											

				when stTest1 =>
					if TstIn = "000100010" then
						stNext <= stTest2;
					else
						stNext <= stFail;
					end if;	

				when stTest2 =>
					if TstIn = "001000100" then
						stNext <= stTest3;
					else
						stNext <= stFail;
					end if;

				when stTest3 =>
				 	if TstIn = "010001001" then
						stNext <= stTest4;
					else
						stNext <= stFail;
					end if;

				when stTest4 =>
				 	if TstIn = "100010001" then
						stNext <= stPass;
					else
						stNext <= stFail;
					end if;

				when stPass =>
 				 	if ctrlRst = '0' then
						stNext <= stPass;
					else
						stNext <= stInit;
					end if;

				when stFail =>	
 				 	if ctrlRst = '0' then
						stNext <= stFail;
					else
						stNext <= stInit;
					end if;
					 	
				-- Some unknown state				
				when others =>
					stNext <= stInit;

			end case;
		end process;


-- This process determines state machine outputs dependant on current state
	process (stCur)
		begin
			case stCur is

				when stInit =>
					TstOut <= "000000000";
					pass <= '0';
					fail <= '0';
											
				when stTest1 =>
					TstOut <= "000100010";
					pass <= '0';
					fail <= '0';

				when stTest2 =>
					TstOut <= "001000100";
					pass <= '0';
					fail <= '0';

				when stTest3 =>
					TstOut <= "010001001";
					pass <= '0';
					fail <= '0';

				when stTest4 =>
					TstOut <= "100010001";
					pass <= '0';
					fail <= '0';

				when stPass =>
					TstOut <= "000000000";
					pass <= '1';
					fail <= '0';

				when stFail =>
					TstOut <= "000000000";
					pass <= '0';
					fail <= '1';
					 	
				-- Some unknown state				
				when others =>
					TstOut <= "000000000";
					pass <= '0';
					fail <= '0';

			end case;
		end process;

end Behavioral;

